//****************************************************
//***** AITENDO OLED WIRELESS MUSIC BOX **************
//*************************** By Minarai Shokunin ****
//*************************** 2013/10/26          ****
//****************************************************

#include "p24FJ64GA002.h"
#include "oled.h"
#include "musicdata.h"

extern unsigned char VirtualScreen[8][128];
extern const unsigned char music1_p1[];
extern const unsigned char music1_p2[];
extern const unsigned char music2_p1[];
extern const unsigned char music2_p2[];
extern const unsigned char music3_p1[];
extern const unsigned char music3_p2[];

// ************ prototypes ***************************
extern void OLED_init();
extern void WriteData(int,int);
extern void Copy2OLED(void);
extern void GR_FILL(unsigned char);
extern void GR_PSET(int,int,unsigned char);
extern void GR_LINE(int,int,int,int,unsigned char);
extern void GR_PUTCH(int,int,unsigned char, unsigned char);
extern void GR_PUTNUM(int,int,int, unsigned char);
extern void GR_PUTSTR(int,int,unsigned char *, unsigned char);


void wait_some(int);
void wait_long(int);
int check_keys(void);

// ************ CONFIGRATION BITS ********************
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF )
_CONFIG2(IESO_OFF & FNOSC_FRCPLL & FCKSM_CSDCMD & OSCIOFNC_OFF & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE)

int sound_enabled1;			// Sound enable flag
int sound_enabled2;			// Sound enable flag

long sound_left1;				// left count for current tone
int sound_tone1;				// current tone
unsigned char *sound_pointer1;			// current tone/len data

long sound_left2;				// left count for current tone
int sound_tone2;				// current tone
unsigned char *sound_pointer2;			// current tone/len data

#define KEY_ON 		1
#define KEY_OFF 	0
#define KEY_PUSHED	2
#define KEY_L 		0
#define KEY_R 		1
#define KEY_S 		2

int key_status[3];
int key_pushed;
int music_no;

// *********************************************
// ************ MAIN ***************************
// *********************************************

int main(void)
{
	int i;
	int count;

	// ************ CPU Initialize **********

	_RCDIV = 0;							// PostScaler 32MHz
	TRISB = 0b0000011100000000;			// PORTB 15-11,7-0 = output  ; 10-9 = input ;
	CNPU1 = 0xffff;						// Set pull-ups
	CNPU2 = 0xffff;						// Set pull-ups

	// *********** Sound Itinialize ***************

	sound_enabled1 = NO;
	sound_left1 = 0;
	sound_enabled2 = NO;
	sound_left2 = 0;

	RPOR0bits.RP0R 		= 18;   // OSC1 ->RB0
    PR2             	= 24; 	// Timer2 period  = 638KHz 
    T2CONbits.TCKPS 	= 0;    // Timer2 Pre scaler 1:1
    T2CONbits.TON   	= 0;    // Timer2 OFF
    OC1CONbits.OCTSEL 	= 0; 
    OC1CONbits.OCM    	= 5;      
    OC1R              	= PR2;   
    OC1RS             	= PR2/2; 

	// *********** Variables initialize *******************

	key_pushed = 0;

	// *********** Display Itinialize ***************

	OLED_init();
	wait_some(1000);
	GR_FILL(0);
	GR_LINE(0,0,127,0,1);
	GR_LINE(0,63,127,63,1);
	GR_LINE(0,0,0,63,1);
	GR_LINE(127,0,127,63,1);
	GR_PUTSTR(10,16,"WIRELESS",1);
	GR_PUTSTR(10,24,"MUSICBOX",1);
	GR_PUTSTR(10,40,"PUSH L",1);
	Copy2OLED();
	while (SW_L == 1) {
	}	
	while (SW_L == 0) {
	}	

	// *********** sound init ***************

 	PR1 = 80*5;             	// 20KHz
    T1CONbits.TCKPS 	= 0;    // Timer1 Pre scaler
 	IEC0bits.T1IE = 1;  // Timer1 interrupt enable
	sound_pointer1 = music1_p1;
	sound_pointer2 = music1_p2;
	sound_left1 =0;
	sound_left2 =0;
	music_no = 0;
	sound_enabled1 = NO;
	sound_enabled2 = NO;

	// ***********  MAIN LOOP **************

	music_no = 0;
	
	while (1) {
		if ((sound_enabled1==0)&&(sound_enabled2==0)) {
    		T2CONbits.TON   	= 0;    // Timer2 OFF
    		T1CONbits.TON   	= 0;    // Timer1 OFF
			wait_long(30);
			GR_FILL(0);
			GR_LINE(0,0,127,0,1);
			GR_LINE(0,63,127,63,1);
			GR_LINE(0,0,0,63,1);
			GR_LINE(127,0,127,63,1);
			GR_PUTSTR(30,16,"NADA SOUSOU",1);
			GR_PUTSTR(30,32,"HANAMIZUKI",1);
			GR_PUTSTR(30,48,"AVE MARIA",1);
			GR_PUTSTR(20,music_no*16+16,">",1);
			Copy2OLED();

			switch (music_no) {
				case 0:
					sound_pointer1 = music1_p1;
					sound_pointer2 = music1_p2;
					break;
				case 1:
					sound_pointer1 = music2_p1;
					sound_pointer2 = music2_p2;
					break;
				case 2:
					sound_pointer1 = music3_p1;
					sound_pointer2 = music3_p2;
					break;
				default:
					break;
			}
			music_no ++;
			if (music_no >2) {music_no =0;}
			sound_enabled1 = YES;
			sound_enabled2 = YES;
			sound_left1 =0;
			sound_left2 =0;
    		T2CONbits.TON   	= 1;    // Timer2 ON
    		T1CONbits.TON   	= 1;    // Timer1 ON
		}
	}
}

// *********************************************
// ************ Sub routines    ****************
// *********************************************

int check_keys(void) {
	int ret;
	static int sw_on = NO;

	ret = 0;

	if ((key_status[0]==KEY_ON)&&(key_status[1]==KEY_ON)) {
		return ret;
	} 

	if ((key_pushed == 0)&&(sw_on == NO)) {
		if ((key_status[0]==KEY_ON)||(key_status[1]==KEY_ON)||(key_status[2]==KEY_ON)) {
			if 	(key_status[0]==KEY_ON) { key_pushed = 1;}		
			if 	(key_status[1]==KEY_ON) { key_pushed |= 2;}	
			if 	(key_status[2]==KEY_ON) { key_pushed = 4;}
			sw_on = YES;
		}
	}else{
		if ((key_status[0]==KEY_OFF)&&(key_status[1]==KEY_OFF)&&(key_status[2]==KEY_OFF)) {
			sw_on = NO;
		}
	}
	return ret;
}

//********** Simple wait loop *************
void wait_some(int t)
{
	int i,j;
	for (i=0;i